library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library proc_common_v3_00_a;
use proc_common_v3_00_a.proc_common_pkg.all;

entity user_logic is
generic
(
    C_NUM_REG                      : integer              := 5;
    C_SLV_DWIDTH                   : integer              := 32
    );
    port
    (
    CS                             : out std_logic;
    D0                             : out std_logic;
    D1                             : out std_logic;
    SCLK                           : out std_logic;

    Bus2IP_Clk                     : in  std_logic;
    Bus2IP_Resetn                  : in  std_logic;
    Bus2IP_Data                    : in  std_logic_vector(C_SLV_DWIDTH-1 downto 0);
    Bus2IP_BE                      : in  std_logic_vector(C_SLV_DWIDTH/8-1 downto 0);
    Bus2IP_RdCE                    : in  std_logic_vector(C_NUM_REG-1 downto 0);
    Bus2IP_WrCE                    : in  std_logic_vector(C_NUM_REG-1 downto 0);
    IP2Bus_Data                    : out std_logic_vector(C_SLV_DWIDTH-1 downto 0);
    IP2Bus_RdAck                   : out std_logic;
    IP2Bus_WrAck                   : out std_logic;
    IP2Bus_Error                   : out std_logic
    );

    attribute MAX_FANOUT : string;
    attribute SIGIS : string;

    attribute SIGIS of Bus2IP_Clk    : signal is "CLK";
    attribute SIGIS of Bus2IP_Resetn : signal is "RST";

    end entity user_logic;

    architecture IMP of user_logic is

    signal slv_reg0                       : std_logic_vector(C_SLV_DWIDTH-1 downto 0);
    signal slv_reg1                       : std_logic_vector(C_SLV_DWIDTH-1 downto 0);
    signal slv_reg2                       : std_logic_vector(C_SLV_DWIDTH-1 downto 0);
    signal slv_reg3                       : std_logic_vector(C_SLV_DWIDTH-1 downto 0);
    signal slv_reg4                       : std_logic_vector(C_SLV_DWIDTH-1 downto 0);
    signal slv_reg_write_sel              : std_logic_vector(4 downto 0);
    signal slv_reg_read_sel               : std_logic_vector(4 downto 0);
    signal slv_ip2bus_data                : std_logic_vector(C_SLV_DWIDTH-1 downto 0);
    signal slv_read_ack                   : std_logic;
    signal slv_write_ack                  : std_logic;

    signal Data0_reg                      : std_logic_vector(C_SLV_DWIDTH-1 downto 0);
    signal Data1_reg                      : std_logic_vector(C_SLV_DWIDTH-1 downto 0);
    signal Control0_reg                   : std_logic_vector(C_SLV_DWIDTH-1 downto 0);
    signal Control1_reg                   : std_logic_vector(C_SLV_DWIDTH-1 downto 0);
    signal Start_reg                      : std_logic_vector(C_SLV_DWIDTH-1 downto 0);
    signal START                          : std_logic;

    type states is (Idle, SendData, SyncData);
    signal CurrentState   : states;
    signal NextState      : states;
    signal ClkDiv         : std_logic;
    signal ClkCnt         : std_logic_vector(31 downto 0);
    signal ShiftCnt       : std_logic_vector(3 downto 0);
    signal EnShiftCounter : std_logic;
    signal EnParalelLoad  : std_logic;
    signal temp0         : std_logic_vector(15 downto 0);
    signal temp1         : std_logic_vector(15 downto 0);     

    begin
    Data0_reg       <= slv_reg0;
    Data1_reg       <= slv_reg1;
    Control0_reg    <= slv_reg2;
    Control1_reg    <= slv_reg3;
    Start_reg       <= slv_reg4;
    
    START           <= Start_reg(0);
    --USER logic implementation added here
    ------------------------------------------
    -- Clock Divider                         -
    --                                       -
    -- Divide AXI BUS Clock to 12.5 MHz      -
    ------------------------------------------
    ClockDivide : process(Bus2IP_Resetn, Bus2IP_Clk)
    begin
        if (Bus2IP_Resetn = '0') then
            ClkCnt <= x"00000000";
        elsif (Bus2IP_Clk = '1' and Bus2IP_Clk'event) then
            ClkCnt <= ClkCnt + '1';
        end if;
    end process;
    
    GenSCLK: process(Bus2IP_Clk)
    begin
        if(Bus2IP_Clk = '1' and Bus2IP_Clk'event) then
            SCLK <= not ClkCnt(4);--2
            ClkDiv <= ClkCnt(4);--2
        end if;
    end process;
    
    ------------------------------------------
    -- Load temporary registers with data    -
    ------------------------------------------
    Load : process(ClkDiv, EnParalelLoad, EnShiftCounter)
    begin
        if(ClkDiv = '1' and ClkDiv'event) then
            if (EnParalelLoad = '1') then
                ShiftCnt <= "0000";
                temp0 <= Control0_reg(7 downto 0) & Data0_reg(7 downto 0);
                temp1 <= Control1_reg(7 downto 0) & Data1_reg(7 downto 0);
            elsif (EnShiftCounter = '1') then
                temp0 <= temp0(14 downto 0) & temp0(15);
                temp1 <= temp1(14 downto 0) & temp1(15);
                ShiftCnt <= ShiftCnt + '1';
            end if;
        end if;
    end process;
    D0 <= temp0(15);
    D1 <= temp1(15);
    
    ------------------------------------------
    -- Finite State Machine                  -
    ------------------------------------------
    SYNC_PROC: process(ClkDiv, Bus2IP_Resetn)
    begin
        if(ClkDiv = '1' and ClkDiv'event) then
            if(Bus2IP_Resetn = '0') then
                CurrentState <= Idle;
            else
                CurrentState <= NextState;
            end if;
        end if;
    end process;
    
    OUTPUT_DECODE: process(CurrentState)
    begin
        if (CurrentState = Idle) then
            EnShiftCounter <= '0';
            EnParalelLoad  <= '1';
            CS             <= '1';
        elsif (CurrentState = SendData) then
            EnShiftCounter <= '1';
            EnParalelLoad  <= '0';
            CS             <= '0';
        elsif (CurrentState = SyncData) then
            EnShiftCounter <= '0';
            EnParalelLoad  <= '0';
            CS             <= '1';
        end if;
    end process;
    
    NEXT_STATE_DECODE: process(CurrentState, ShiftCnt, START)
    begin
    
        NextState <= CurrentState;
        case(CurrentState) is
            when Idle =>
                if(START = '1') then
                    NextState <= SendData;
                end if;
            when SendData =>
                if(ShiftCnt = "1111") then
                    NextState <= SyncData;
                end if;
            when SyncData =>    
                if(START = '0') then
                    NextState <= Idle;
                end if;
            when others => NextState <= Idle;
         end case;
    end process;
    -- Example code to read/write user logic slave model s/w accessible registers
    -- 
    -- Note:
    -- The example code presented here is to show you one way of reading/writing
    -- software accessible registers implemented in the user logic slave model.
    -- Each bit of the Bus2IP_WrCE/Bus2IP_RdCE signals is configured to correspond
    -- to one software accessible register by the top level template. For example,
    -- if you have four 32 bit software accessible registers in the user logic,
    -- you are basically operating on the following memory mapped registers:
    -- 
    --    Bus2IP_WrCE/Bus2IP_RdCE   Memory Mapped Register
    --                     "1000"   C_BASEADDR + 0x0
    --                     "0100"   C_BASEADDR + 0x4
    --                     "0010"   C_BASEADDR + 0x8
    --                     "0001"   C_BASEADDR + 0xC
    -- 
    ------------------------------------------
    slv_reg_write_sel <= Bus2IP_WrCE(4 downto 0);
    slv_reg_read_sel  <= Bus2IP_RdCE(4 downto 0);
    slv_write_ack     <= Bus2IP_WrCE(0) or Bus2IP_WrCE(1) or Bus2IP_WrCE(2) or Bus2IP_WrCE(3) or Bus2IP_WrCE(4);
    slv_read_ack      <= Bus2IP_RdCE(0) or Bus2IP_RdCE(1) or Bus2IP_RdCE(2) or Bus2IP_RdCE(3) or Bus2IP_RdCE(4);

    -- implement slave model software accessible register(s)
    SLAVE_REG_WRITE_PROC : process( Bus2IP_Clk ) is
    begin

    if Bus2IP_Clk'event and Bus2IP_Clk = '1' then
      if Bus2IP_Resetn = '0' then
        slv_reg0 <= (others => '0');
        slv_reg1 <= (others => '0');
        slv_reg2 <= (others => '0');
        slv_reg3 <= (others => '0');
        slv_reg4 <= (others => '0');
      else
        case slv_reg_write_sel is
          when "10000" =>
            for byte_index in 0 to (C_SLV_DWIDTH/8)-1 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                slv_reg0(byte_index*8+7 downto byte_index*8) <= Bus2IP_Data(byte_index*8+7 downto byte_index*8);
              end if;
            end loop;
          when "01000" =>
            for byte_index in 0 to (C_SLV_DWIDTH/8)-1 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                slv_reg1(byte_index*8+7 downto byte_index*8) <= Bus2IP_Data(byte_index*8+7 downto byte_index*8);
              end if;
            end loop;
          when "00100" =>
            for byte_index in 0 to (C_SLV_DWIDTH/8)-1 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                slv_reg2(byte_index*8+7 downto byte_index*8) <= Bus2IP_Data(byte_index*8+7 downto byte_index*8);
              end if;
            end loop;
          when "00010" =>
            for byte_index in 0 to (C_SLV_DWIDTH/8)-1 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                slv_reg3(byte_index*8+7 downto byte_index*8) <= Bus2IP_Data(byte_index*8+7 downto byte_index*8);
              end if;
            end loop;
          when "00001" =>
            for byte_index in 0 to (C_SLV_DWIDTH/8)-1 loop
              if ( Bus2IP_BE(byte_index) = '1' ) then
                slv_reg4(byte_index*8+7 downto byte_index*8) <= Bus2IP_Data(byte_index*8+7 downto byte_index*8);
              end if;
            end loop;
          when others => null;
        end case;
      end if;
    end if;

    end process SLAVE_REG_WRITE_PROC;

    -- implement slave model software accessible register(s) read mux
    SLAVE_REG_READ_PROC : process( slv_reg_read_sel, slv_reg0, slv_reg1, slv_reg2, slv_reg3, slv_reg4 ) is
    begin

    case slv_reg_read_sel is
      when "10000" => slv_ip2bus_data <= slv_reg0;
      when "01000" => slv_ip2bus_data <= slv_reg1;
      when "00100" => slv_ip2bus_data <= slv_reg2;
      when "00010" => slv_ip2bus_data <= slv_reg3;
      when "00001" => slv_ip2bus_data <= slv_reg4;
      when others => slv_ip2bus_data <= (others => '0');
    end case;

    end process SLAVE_REG_READ_PROC;

    ------------------------------------------
    -- Example code to drive IP to Bus signals
    ------------------------------------------
    IP2Bus_Data  <= slv_ip2bus_data when slv_read_ack = '1' else
                  (others => '0');

    IP2Bus_WrAck <= slv_write_ack;
    IP2Bus_RdAck <= slv_read_ack;
    IP2Bus_Error <= '0';

end IMP;
